/*
 * This file is part of essential (http://essential.craftforge.net).
 *
 *     Essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2011 Christian Bick.
 */

package net.craftforge.essential.supply;

import net.craftforge.essential.controller.ControllerException;
import net.craftforge.essential.controller.Configuration;
import net.craftforge.essential.controller.annotations.Consumes;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * De-serializes strings to objects.
 *
 * @author Christian Bick
 * @since 02.12.2010
 */
public class Consumer {

    protected Configuration config;

    public Consumer(Configuration config) {
        this.config = config;    
    }

    /**
     * De-serializes the input in plain text format to an object of the specified type.
     *
     * @param type The type
     * @param input The input
     * @return The object
     * @throws net.craftforge.essential.controller.ControllerException Failed to find a convenient method to initialize object from input
     * @throws InstantiationException Failed to instantiate object from constructor
     * @throws InvocationTargetException Failed to invoke 'valueOf' method
     * @throws IllegalAccessException Failed to invoke constructor or 'valueOf' method
     */
    @Consumes({"text/plain"})
    public Object plain(Class<?> type, String input) throws ControllerException, InstantiationException, InvocationTargetException, IllegalAccessException {
        if (type.equals(boolean.class)) {
            type = Boolean.class;
        } else if (type.equals(byte.class)) {
            type = Byte.class;
        } else if (type.equals(short.class)) {
            type = Short.class;
        } else if (type.equals(int.class)) {
            type = Integer.class;
        } else if (type.equals(long.class)) {
            type = Long.class;
        } else if (type.equals(double.class)) {
            type = Double.class;
        } else if (type.equals(float.class)) {
            type = Float.class;
        } else if (type.equals(char.class)) {
            type = Character.class;
        }
        try {
            Constructor constructor = type.getConstructor(new Class[] {String.class});
            return constructor.newInstance(input);
        } catch (NoSuchMethodException e) {
            try {
                Method method = type.getMethod("valueOf", new Class[] {String.class});
                return method.invoke(null, input);
            } catch (NoSuchMethodException e1) {
                throw new ControllerException("Unable to find a convenient method to initialize object from input");
            }
        }
    }
}
