/*
 * This file is part of essential (http://essential.craftforge.net).
 *
 *     Essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2011 Christian Bick.
 */

package net.craftforge.essential.supply;

import net.craftforge.essential.controller.ControllerException;
import net.craftforge.essential.controller.Configuration;
import net.craftforge.essential.controller.annotations.Produces;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Provides an implementation of the Controllable interface in
 * a view setup.
 *
 * @author Christian Bick
 * @since 02.12.2010
 */
@SuppressWarnings("unused")
public class Producer {

    /**
     * The configuration
     */
    protected Configuration config;

    /**
     * Constructs the producer by supplying the controller configuration.
     *
     * @param config The configuration
     */
    public Producer(Configuration config) {
        this.config = config;    
    }

    /**
     * Serializes the result object to plain text, using the specified character encoding.
     * Uses the output stream for streaming.
     *
     * @param result The result
     * @param out The output stream
     * @param encoding The character encoding
     * @throws IOException Failed to initialize the writer for the output stream
     * @throws ControllerException Failed to serialize object from toString()
     */
    @Produces({"text/plain"})
    public void plain(Object result, OutputStream out, String encoding) throws IOException, ControllerException {
        try {
            Method method = result.getClass().getMethod("toString");
            OutputStreamWriter writer = new OutputStreamWriter(out, encoding);
            writer.write((String)method.invoke(result));
            writer.flush();
        } catch (NoSuchMethodException e) {
            throw new ControllerException("Unable to serialize object from toString()", e);
        } catch (IllegalAccessException e) {
            throw new ControllerException("Unable to serialize object from toString()", e);
        } catch (InvocationTargetException e) {
            throw new ControllerException("Unable to serialize object from toString()", e);
        }
    }

}
